// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Arrival;
import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.JointArrDep;
import com.sncf.checker.bo.JointArrDepEntry;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.ArrivalDAO;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.DepartureDAO;
import com.sncf.checker.io.TrainDAO;

/**
 * Assembled train category compatibility.
 *
 * When two trains are assigned to departures belonging to a joint-departure,
 * the categories of the assigned trains must be compatible with each other.
 *
 * @author YDS
 *
 */
public class AssembledTrainCategoryCompatibilityConstraint implements
        Constraint {

    /** Interface to Data layer. */
    private final DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public AssembledTrainCategoryCompatibilityConstraint(
            final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public final boolean validate() throws ROADEFException {
        ArrivalDAO arrivalDAO = factory.getArrivalDAO();
        DepartureDAO departureDAO = factory.getDepartureDAO();
        TrainDAO trainDAO = factory.getTrainDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        for (Train train : trainDAO.loadAll()) {
            Fill departureFill = null;
            ArrayList<String> assembledTrains = new ArrayList<String>();
            Arrival arrival = train.getArrival();
            if (arrival != null) {
                JointArrDep joint = arrival.getJointArrDep();
                if (joint != null) {
                    for (JointArrDepEntry entry : joint.getEntryList()) {
                        Arrival jointArrival =
                                arrivalDAO.loadByName(entry.getArrDep());
                        assembledTrains.add(jointArrival.getTrain().getName());
                    }
                } else {
                    assembledTrains.add(train.getName());
                }
            } else {
                assembledTrains.add(train.getName());
            }

            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals("EndJunction")
                        || fill.getEvent().equals("EndDisjunction")) {
                    assembledTrains = new ArrayList<String>(
                            Arrays.asList(fill.getComplement().split("\\+")));
                }
                if (fill.getEvent().equals("Departure")) {
                    departureFill = fill;
                    break;
                }
            }

            if (assembledTrains.size() > 1 && departureFill != null) {
                Departure departure = departureDAO.loadByName(
                        departureFill.getComplement());
                if (departure != null) {
                    if (departure.getJointArrDep() != null) {
                        String category = train.getCategory().getCatGroup();
                        for (String t : assembledTrains) {
                            Train other = trainDAO.loadByName(t);
                            if (other != null) {
                                if (!category.equals(
                                        other.getCategory().getCatGroup())) {
                                    logger.log(Level.INFO, "Assembled train "
                                            + "category compatibility error : "
                                            + train.getName() + " (" + category
                                            + ") is not compatible with "
                                            + other.getName() + " ("
                                            + other.getCategory().getCatGroup()
                                            + ").");
                                    valid = false;
                                }
                            }
                        }
                    }
                }
            }
        }
        return valid;
    }

}
