// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.ArrDepSequence;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Platform;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.TrainDAO;

/**
 * Entrance in the system.
 *
 * @author YDS
 *
 */
public class EntranceConstraint implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public EntranceConstraint(final DAOFactory factory)  {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public final boolean validate() throws ROADEFException {
        TrainDAO dao = factory.getTrainDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = dao.loadAll();
        for (Train train : trainList) {
            // train initially in the system
            if (train.getResource() != null) {
                List<Fill> fillList = train.getFill();
                Iterator<Fill> itr = fillList.iterator();
                Fill fill = null;
                if (!itr.hasNext()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " must begin by an EnterSystem "
                        + "event (schedule missing).");
                    valid = false;
                    continue;
                }
                fill = itr.next();
                if (!fill.getEvent().equals("EnterSystem")) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " must begin by an EnterSystem "
                        + "event (" + fill.getEvent() + " found).");
                    valid = false;
                    continue;
                }
                if (fill.getResource() != train.getResource()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " EnterSystem event must occur "
                        + "on " + train.getResource().getName() + " ("
                        + fill.getResource().getName() + " found).");
                    valid = false;
                    continue;
                }
                if (fill.getDate().getTime() != Datetime.MIN.getTime()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " EnterSystem event must occur "
                        + "at " + Datetime.MIN.toString() + " ("
                        + fill.getDate().toString() + " found).");
                    valid = false;
                    continue;
                }
                if (!itr.hasNext()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " EnterSystem event must be "
                        + "followed by an EnterResource event "
                        + "(schedule missing).");
                    valid = false;
                    continue;
                }
                fill = itr.next();
                if (!fill.getEvent().equals("EnterResource")) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " EnterSystem event must be "
                        + "followed by an EnterResource event ("
                        + fill.getEvent() + " found).");
                    valid = false;
                    continue;
                }
                if (fill.getResource() != train.getResource()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " first EnterResource event must "
                        + "occur on " + train.getResource().getName() + " ("
                        + fill.getResource().getName() + " found).");
                    valid = false;
                    continue;
                }
                if (fill.getDate().getTime() != Datetime.MIN.getTime()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " first EnterRessource event must "
                        + "occur at " + Datetime.MIN.toString() + " ("
                        + fill.getDate().toString() + " found).");
                    valid = false;
                }

            // train associated with an arrival
            } else {
                List<Fill> fillList = train.getFill();
                Fill fill = null;
                List<ArrDepSequence> arrDepSequenceList = train.getArrival().getArrSequence();
                Collections.sort(arrDepSequenceList, new SortArrDepSequenceByOrder());
                ArrDepSequence sequence = null;
                Iterator<ArrDepSequence> itrSequence =
                        arrDepSequenceList.iterator();
                Iterator<Fill> itr = fillList.iterator();
                if (!itr.hasNext()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " must begin by an EnterSystem "
                        + "event (schedule missing).");
                    valid = false;
                    continue;
                }
                fill = itr.next();
                sequence = train.getArrival().getArrSequence().get(0);
                if (!fill.getEvent().equals("EnterSystem")) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " must begin by an EnterSystem "
                        + "event (" + fill.getEvent() + " found).");
                    valid = false;
                    continue;
                }
                if (fill.getResource() != sequence.getTrackGroup()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " EnterSystem event must occur "
                        + "on " + sequence.getTrackGroup().getName() + " ("
                        + fill.getResource().getName() + " found).");
                    valid = false;
                    continue;
                }
                boolean isSequenceValid = true;
                while (itrSequence.hasNext()) {
                    sequence = itrSequence.next();
                    if (!itr.hasNext()) {
                        logger.log(Level.INFO, "Entrance in the system error : "
                            + train.getName() + " EnterResource event expected"
                            + " (schedule missing).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    fill = itr.next();
                    if (!fill.getEvent().equals("EnterResource")) {
                        logger.log(Level.INFO, "Entrance in the system error : "
                            + train.getName() + " EnterResource expected ("
                            + fill.getEvent() + " at "
                            + fill.getDate().toString() + " found).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    if (fill.getResource() != sequence.getTrackGroup()) {
                        logger.log(Level.INFO, "Entrance in the system error : "
                            + train.getName() + " EnterResource event must "
                            + "occur on " + sequence.getTrackGroup().getName()
                            + " (" + fill.getResource().getName() + " at "
                            + fill.getDate().toString() + " found).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    if (!itr.hasNext()) {
                        logger.log(Level.INFO, "Entrance in the system error : "
                            + train.getName() + " ExitResource expected "
                            + "(schedule missing).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    fill = itr.next();
                    if (!fill.getEvent().equals("ExitResource")) {
                        logger.log(Level.INFO, "Entrance in the system error : "
                            + train.getName() + " ExitResource expected "
                            + fill.getEvent() + " at "
                            + fill.getDate().toString() + " found).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    if (fill.getResource() != sequence.getTrackGroup()) {
                        logger.log(Level.INFO, "Entrance in the system error : "
                            + train.getName() + " ExitResource event "
                            + "must occur on " + sequence.getTrackGroup().getName()
                            + " (" + fill.getResource().getName() + " at "
                            + fill.getDate().toString() + " found).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                }
                if (!isSequenceValid) {
                    continue;
                }
                if (!itr.hasNext()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " EnterResource on a platform "
                        + "expected (schedule missing).");
                    valid = false;
                    continue;
                }
                fill = itr.next();
                if (!fill.getEvent().equals("EnterResource")) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " EnterResource expected "
                        + fill.getEvent() + " at "
                        + fill.getDate().toString() + " found).");
                    valid = false;
                    isSequenceValid = false;
                    break;
                }
                if (fill.getResource().getClass() != Platform.class) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " Platform expected ("
                        + fill.getResource().getClass().getName() + " at "
                        + fill.getDate().toString() + " found).");
                    valid = false;
                    continue;
                }
                if (train.getArrival().getArrTime().getTime()
                    != fill.getDate().getTime()) {
                    logger.log(Level.INFO, "Entrance in the system error : "
                        + train.getName() + " Arrival time expected at "
                        + train.getArrival().getArrTime().toString() + " ("
                        + fill.getDate().toString() + " found).");
                    valid = false;
                    continue;
                }
            }
        }

        return valid;
    }
}
