// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.Collections;
import java.util.List;
import java.util.ListIterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.ArrDepSequence;
import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Platform;
import com.sncf.checker.bo.Resource;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.DepartureDAO;
import com.sncf.checker.io.TrainDAO;

/**
 * Exit of the system.
 *
 * @author YDS
 *
 */
public class ExitConstraint implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public ExitConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public final boolean validate() throws ROADEFException {
        TrainDAO trainDAO = factory.getTrainDAO();
        DepartureDAO departureDAO = factory.getDepartureDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = trainDAO.loadAll();
        for (Train train : trainList) {
            Departure departure = null;
            boolean isDepartureCorrect = true;
            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals("Departure")) {
                    departure = departureDAO.loadByName(fill.getComplement());
                    if (departure == null) {
                        logger.log(Level.INFO, "Exit of the system error : "
                            + train.getName() + " has an unknown departure"
                            + " (" + fill.getComplement() + ").");
                        isDepartureCorrect = false;
                        valid = false;

                    }
                    break;
                }
            }
            if (!isDepartureCorrect) {
                continue;
            }
            // train associated with a departure
            if (departure != null)
            {
                List<Fill> fillList = train.getFill();
                Fill fill = null;
                ArrDepSequence sequence = null;
                List<ArrDepSequence> arrDepSequenceList =
                        departure.getDepSequence();
                Collections.sort(arrDepSequenceList
                        , new SortArrDepSequenceByOrder());
                ListIterator<ArrDepSequence> itrSequence =
                        arrDepSequenceList.listIterator(
                        arrDepSequenceList.size());
                ListIterator<Fill> itr = fillList.listIterator(
                        fillList.size());
                if (!itr.hasPrevious()) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " must end with an ExitSystem "
                        + "event (schedule missing).");
                    valid = false;
                    continue;
                }
                fill = itr.previous();
                sequence = departure.getDepSequence().get(
                        departure.getDepSequence().size() - 1);
                if (!fill.getEvent().equals("ExitSystem")) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " must end with an ExitSystem "
                        + "event (" + fill.getEvent() + " found).");
                    valid = false;
                    continue;
                }
                if (fill.getResource() != sequence.getTrackGroup()) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " ExitSystem event must occur "
                        + "on " + sequence.getTrackGroup().getName() + " ("
                        + fill.getResource().getName() + " found).");
                    valid = false;
                    continue;
                }
                boolean isSequenceValid = true;
                while (itrSequence.hasPrevious()) {
                    sequence = itrSequence.previous();
                    if (!itr.hasPrevious()) {
                        logger.log(Level.INFO, "Exit of the system error : "
                            + train.getName() + " ExitResource event expected"
                            + " (schedule missing).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    fill = itr.previous();
                    if (!fill.getEvent().equals("ExitResource")) {
                        logger.log(Level.INFO, "Exit of the system error : "
                            + train.getName() + " ExitResource expected ("
                            + fill.getEvent() + " at "
                            + fill.getDate().toString() + " found).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    if (fill.getResource() != sequence.getTrackGroup()) {
                        logger.log(Level.INFO, "Exit of the system error : "
                            + train.getName() + " ExitResource event must "
                            + "occur on " + sequence.getTrackGroup().getName()
                            + " (" + fill.getResource().getName() + " at "
                            + fill.getDate().toString() + " found).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    if (!itr.hasPrevious()) {
                        logger.log(Level.INFO, "Exit of the system error : "
                            + train.getName() + " EnterResource expected "
                            + "(schedule missing).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    fill = itr.previous();
                    if (!fill.getEvent().equals("EnterResource")) {
                        logger.log(Level.INFO, "Exit of the system error : "
                            + train.getName() + " EnterResource expected ("
                            + fill.getEvent() + " at "
                            + fill.getDate().toString() + " found).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                    if (fill.getResource() != sequence.getTrackGroup()) {
                        logger.log(Level.INFO, "Exit of the system error : "
                            + train.getName() + " EnterResource event "
                            + "must occur on " + sequence.getTrackGroup().getName()
                            + " (" + fill.getResource().getName() + " at "
                            + fill.getDate().toString() + " found).");
                        valid = false;
                        isSequenceValid = false;
                        break;
                    }
                }
                if (!isSequenceValid) {
                    continue;
                }
                if (!itr.hasNext()) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " ExitResource on a platform "
                        + "expected (schedule missing).");
                    valid = false;
                    continue;
                }
                fill = itr.previous();
                if (!fill.getEvent().equals("ExitResource")) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " ExitResource expected ("
                        + fill.getEvent() + " at "
                        + fill.getDate().toString() + " found).");
                    valid = false;
                    isSequenceValid = false;
                    break;
                }
                if (fill.getResource().getClass() != Platform.class) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " Platform expected ("
                        + fill.getResource().getClass().getName() + " at "
                        + fill.getDate().toString() + " found).");
                    valid = false;
                    continue;
                }
                if (departure.getDepTime().getTime()
                    != fill.getDate().getTime()) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " Departure time expected at "
                        + departure.getDepTime().toString() + " ("
                        + fill.getDate().toString() + " found).");
                    valid = false;
                    continue;
                }
            // train left in the system
            } else {
                Resource exitResource = null;
                List<Fill> fillList = train.getFill();
                ListIterator<Fill> itr
                    = fillList.listIterator(fillList.size());
                Fill fill = null;
                if (!itr.hasPrevious()) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " must end with an ExitSystem "
                        + "event (schedule missing).");
                    valid = false;
                    continue;
                }
                fill = itr.previous();
                exitResource = fill.getResource();
                if (!fill.getEvent().equals("ExitSystem")) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " must end with an ExitSystem "
                        + "event (" + fill.getEvent() + " found).");
                    valid = false;
                    continue;
                }
                if (fill.getDate().getTime() != Datetime.MAX.getTime()) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " EnterSystem event must occur "
                        + "occur at " + Datetime.MAX.toString() + " ("
                        + fill.getDate().toString() + " found).");
                    valid = false;
                    continue;
                }
                if (!itr.hasPrevious()) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " ExitSystem event must be "
                        + "preceded by an ExitResource event "
                        + "(schedule missing).");
                    valid = false;
                    continue;
                }
                fill = itr.previous();
                if (!fill.getEvent().equals("ExitResource")) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " ExitSystem event must be "
                        + "preceded by an ExitResource event ("
                        + fill.getEvent() + " found).");
                    valid = false;
                    continue;
                }
                if (fill.getResource() != exitResource) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " last exit event must "
                        + "occur on " + exitResource.getName() + " ("
                        + fill.getResource().getName() + " found).");
                    valid = false;
                    continue;
                }
                if (fill.getDate().getTime() != Datetime.MAX.getTime()) {
                    logger.log(Level.INFO, "Exit of the system error : "
                        + train.getName() + " last ExitRessource event must "
                        + "occur at " + Datetime.MAX.toString() + " ("
                        + fill.getDate().toString() + " found).");
                    valid = false;
                }
            }
        }

        return valid;
    }

}
