// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Facility;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Duration;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.TrainDAO;

/**
 * Train junction / disjunction operation cost.
 *
 * @author YDS
 *
 */
public class MaintenanceOperationsConstraint implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public MaintenanceOperationsConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public boolean validate() throws ROADEFException {
        TrainDAO trainDAO = factory.getTrainDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = trainDAO.loadAll();
        for (Train train : trainList) {
            IState state = new DefaultState();
            for (Fill fill : train.getFill()) {
                 state = state.handle(fill, logger);
            }
            if (!state.isValid()) {
                valid = false;
            }
        }

        return valid;
    }

    /**
     * state pattern.
     *
     * @author YDS
     *
     */
    private interface IState {

        /**
         * Process a Fill object.
         *
         * @param fill Fill to process
         * @param logger log
         * @return next state
         */
        IState handle(final Fill fill, final Logger logger);
        /**
         * Tells if the process is in a valid state.
         *
         * @return true if valid, false otherwise
         */
        boolean isValid();
    }

    /**
     * default state.
     *
     * @author YDS
     *
     */
    private class DefaultState implements IState {
        /** number of D maintenance */
        private int maintenanceD;
        /** number of T maintenance */
        private int maintenanceT;
        /** Validity of the solution. */
        private boolean valid;

        /**
         * Constructor.
         */
        public DefaultState() {
            maintenanceD = 0;
            maintenanceT = 0;
            valid = true;
        }

        /**
         * Constructor.
         *
         * @param d number of D maintenance
         * @param t number of T maintenance
         * @param valid validity
         */
        public DefaultState(final int d, final int t, final boolean valid) {
            maintenanceD = d;
            maintenanceT = t;
            this.valid = valid;
        }

        /** {@inheritDoc} */
        public IState handle(final Fill fill, final Logger logger)
        {
            if (fill.getEvent().equals("EnterResource")
                    && fill.getResource().getClass() == Facility.class) {
                return new FacilityState(maintenanceD, maintenanceT, valid);
            } else if (fill.getEvent().equals("BegMaintenance")) {
                logger.log(Level.INFO, "Maintenance operations error : "
                        + fill.getTrain().getName()
                        + " BegMaintenance event must "
                        + "occur after an EnterResource event on a "
                        + "maintenance facility at "
                        + fill.getDate().toString() + ".");
                valid = false;
            } else if (fill.getEvent().equals("EndMaintenance")) {
                logger.log(Level.INFO, "Maintenance operations error : "
                        + fill.getTrain().getName() + " EndMaintenance event "
                        + "must occur after a BegMaintenance event "
                        + "on a maintenance facility at "
                        + fill.getDate().toString() + ".");
                valid = false;
            }
            return this;
        }

        /** {@inheritDoc} */
        public boolean isValid() {
            return valid;
        }
    }

    /**
     * train on a facility.
     *
     * @author YDS
     *
     */
    private class FacilityState implements IState {
        /** number of D maintenance */
        private int maintenanceD;
        /** number of T maintenance */
        private int maintenanceT;
        /** Validity of the solution. */
        private boolean valid;

        /**
         * Constructor.
         *
         * @param d number of D maintenance
         * @param t number of T maintenance
         * @param valid validity
         */
        public FacilityState(int d, int t, boolean valid) {
            maintenanceD = d;
            maintenanceT = t;
            this.valid = valid;
        }

        /** {@inheritDoc} */
        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals("BegMaintenance")) {
                if (!(fill.getComplement().equals("D")
                        || fill.getComplement().equals("T"))) {
                    logger.log(Level.INFO, "Maintenance operations error : "
                            + fill.getTrain().getName()
                            + " has an unknown operation type at "
                            + fill.getDate().toString()
                            + ": \"" + fill.getComplement() + "\".");
                }
                return new MaintenanceState(maintenanceD, maintenanceT, fill
                        , valid);
            } else if (fill.getEvent().equals("ExitResource")) {
                return new DefaultState(maintenanceD, maintenanceT, valid);
            } else if (fill.getEvent().equals("EndMaintenance")) {
                logger.log(Level.INFO, "Maintenance operations error : "
                        + fill.getTrain().getName() + " EndMaintenance event "
                        + "must occur after a BegMaintenance event at "
                        + fill.getDate().toString() + ".");
                valid = false;
            }
            return this;
        }

        /** {@inheritDoc} */
        public boolean isValid() {
            return valid;
        }
    }

    /**
     * train in maintenance.
     *
     * @author YDS
     *
     */
    private class MaintenanceState implements IState {
        /** maintenance beginning */
        private Fill begin;
        /** number of D maintenance */
        private int maintenanceD;
        /** number of T maintenance */
        private int maintenanceT;
        /** Validity of the solution. */
        private boolean valid;

        /**
         * Constructor.
         *
         * @param d number of D maintenance
         * @param t number of T maintenance
         * @param fill maintenance beginning
         * @param valid validity
         */
        public MaintenanceState(final int d, final int t, final Fill fill
                , boolean valid) {
            begin = fill;
            maintenanceD = d;
            maintenanceT = t;
            this.valid = valid;
        }

        /** {@inheritDoc} */
        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals("EndMaintenance")) {
                Duration expectedDuration = null;
                if (fill.getComplement().equals("D")
                        && fill.getComplement().equals(begin.getComplement())) {
                    maintenanceD++;
                    if (maintenanceD > 1) {
                        logger.log(Level.INFO, "Maintenance operations error : "
                                + fill.getTrain().getName()
                                + " must not perform more than one"
                                + " operation of type D at "
                                + fill.getDate().toString());
                        valid = false;
                    } else {
                        expectedDuration = fill.getTrain().getCategory().getMaintTimeD();
                    }
                } else if (fill.getComplement().equals("T")
                        && fill.getComplement().equals(begin.getComplement())) {
                    maintenanceT++;
                    if (maintenanceT > 1) {
                        logger.log(Level.INFO, "Maintenance operations error : "
                                + fill.getTrain().getName()
                                + " must not perform more than one"
                                + " operation of type T at "
                                + fill.getDate().toString());
                        valid = false;
                    } else {
                        expectedDuration = fill.getTrain().getCategory().getMaintTimeT();
                    }
                } else if (!fill.getComplement().equals(begin.getComplement())) {
                    logger.log(Level.INFO, "Maintenance operations error : "
                            + fill.getTrain().getName()
                            + " BegMaintenance event and EndMaintenance "
                            + "event don't have the same operation type at "
                            + fill.getDate().toString()
                            + " (BegMaintenance \"" + begin.getComplement()
                            + "\" EndMaintenance \""
                            + fill.getComplement() + "\").");
                }
                int duration = fill.getDate().getTime()
                        - begin.getDate().getTime();
                if (expectedDuration != null) {
                    if (duration != expectedDuration.getTime()) {
                        logger.log(Level.INFO, "Maintenance operations error : "
                                + fill.getTrain().getName()
                                + " maintenance time at "
                                + fill.getDate().toString()
                                + " should be " + expectedDuration.toString()
                                + " (" + new Duration(duration).toString()
                                + " found).");
                        valid = false;
                    }
                }
            } else {
                logger.log(Level.INFO, "Maintenance operations error : "
                        + fill.getTrain().getName()
                        + " EndMaintenance event expected"
                        + " (" + fill.getEvent() + " at "
                        + fill.getDate().toString() + " found).");
                valid = false;
            }
            return new FacilityState(maintenanceD, maintenanceT, valid);
        }

        /** {@inheritDoc} */
        public boolean isValid() {
            return valid;
        }
    }
}
