// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Arrival;
import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Platform;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.TrainDAO;

/**
 * Platform usage cost.
 *
 * @author YDS
 *
 */
public class PlatformUsageCost implements Cost {

    private DAOFactory factory;

    public PlatformUsageCost(final DAOFactory factory) {
        this.factory = factory;
    }

    public float calculate() throws ROADEFException {
        TrainDAO trainDAO = factory.getTrainDAO();
        float score = 0.0f;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = trainDAO.loadAll();
        for (Train train : trainList) {
            IState state = new DefaultState(factory);
            for (Fill fill : train.getFill()) {
                 state = state.handle(fill, logger);
            }
            score += state.getScore();
        }

        return score;
    }

    private interface IState {

        IState handle(final Fill fill, final Logger logger);
        float getScore();
    }

    private class DefaultState implements IState {
        private DAOFactory factory;
        private float score;

        public DefaultState(DAOFactory factory) {
            this.factory = factory;
            score = 0.0f;
        }

        public DefaultState(DAOFactory factory, float score) {
            this.factory = factory;
            this.score = score;
        }

        public IState handle(final Fill fill, final Logger logger)
        {
            if (fill.getEvent().equals("EnterResource") && fill.getResource().getClass() == Platform.class) {
                return new PlatformState(factory, fill, score);
            }
            return this;
        }

        public float getScore() {
            return score;
        }
    }

    private class PlatformState implements IState {
        private DAOFactory factory;
        private Fill begin;
        private float score;

        public PlatformState(DAOFactory factory, Fill fill, float score) {
            this.factory = factory;
            this.begin = fill;
            this.score = score;
        }

        public IState handle(final Fill fill, final Logger logger)
        {
            if (fill.getEvent().equals("ExitResource")) {
                return new DefaultState(factory, score);
            } else if (fill.getEvent().equals("Arrival")) {
                return new ArrivalState(factory, begin, score);
            } else if (fill.getEvent().equals("Departure")) {
                return new DepartureState(factory, begin, fill, score);
            }
            return this;
        }

        public float getScore() {
            return score;
        }
    }

    private class DepartureState implements IState {
        private DAOFactory factory;
        private Fill begin;
        private Fill departure;
        private float score;

        public DepartureState(DAOFactory factory, final Fill fill, Fill departure, float score) {
            this.factory = factory;
            begin = fill;
            this.score = score;
            this.departure = departure;
        }

        public IState handle(final Fill fill, final Logger logger)
        {
            if (fill.getEvent().equals("ExitResource")) {
                int duration = fill.getDate().getTime() - begin.getDate().getTime();
                Departure dep = factory.getDepartureDAO().loadByName(departure.getComplement());
                if (dep != null) {
                    logger.log(Level.INFO, "Platform usage cost : "
                            + fill.getTrain().getName() + " dwell time is "
                            + duration + " s (ideal dwell time "
                            + dep.getIdealDwell().getTime() + " s).");
                    float dwellCost = factory.getParametersDAO().getParameters().getDwellCost();
                    score += dwellCost * Math.abs(duration - dep.getIdealDwell().getTime());
                }
                return new DefaultState(factory, score);
            }
            return this;
        }

        public float getScore() {
            return score;
        }
    }

    private class ArrivalState implements IState {
        private DAOFactory factory;
        private Fill begin;
        private float score;

        public ArrivalState(DAOFactory factory, final Fill fill, float score) {
            this.factory = factory;
            begin = fill;
            this.score = score;
        }

        public IState handle(final Fill fill, final Logger logger)
        {
            if (fill.getEvent().equals("ExitResource")) {
                int duration = fill.getDate().getTime() - begin.getDate().getTime();
                Arrival arr = fill.getTrain().getArrival();
                if (arr != null) {
                    logger.log(Level.INFO, "Platform usage cost : "
                            + fill.getTrain().getName() + " dwell time is "
                            + duration + " s (ideal dwell time "
                            + arr.getIdealDwell().getTime() + " s).");
                    float dwellCost = factory.getParametersDAO().getParameters().getDwellCost();
                    score += dwellCost * Math.abs(duration - arr.getIdealDwell().getTime());
                }
                return new DefaultState(factory, score);
            } else if (fill.getEvent().equals("Departure")) {
                return new ArrivalDepartureState(factory, begin, fill, score);
            }
            return this;
        }

        public float getScore() {
            return score;
        }
    }

    private class ArrivalDepartureState implements IState {
        private DAOFactory factory;
        private Fill begin;
        private float score;
        private Fill departure;

        public ArrivalDepartureState(DAOFactory factory, final Fill fill, Fill departure, float score) {
            this.factory = factory;
            begin = fill;
            this.score = score;
            this.departure = departure;
        }

        public IState handle(final Fill fill, final Logger logger)
        {
            if (fill.getEvent().equals("ExitResource")) {
                int duration = fill.getDate().getTime() - begin.getDate().getTime();
                Departure dep = factory.getDepartureDAO().loadByName(departure.getComplement());
                if (dep != null) {
                    Arrival arr = fill.getTrain().getArrival();
                    if (arr != null) {
                        int idealDwell = arr.getIdealDwell().getTime() + dep.getIdealDwell().getTime();
                        logger.log(Level.INFO, "Platform usage cost : "
                                + fill.getTrain().getName() + " dwell time is "
                                + duration + " s (ideal dwell time "
                                + idealDwell + " s).");
                        float dwellCost = factory.getParametersDAO().getParameters().getDwellCost();
                        score += dwellCost * Math.abs(duration - idealDwell);
                    }
                }
                return new DefaultState(factory, score);
            }
            return this;
        }

        public float getScore() {
            return score;
        }
    }
}
