// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Arrival;
import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Platform;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Duration;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.TrainDAO;

public class PlatformUseMaximumDurationConstraint implements Constraint {

    private DAOFactory factory;

    public PlatformUseMaximumDurationConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        TrainDAO trainDAO = factory.getTrainDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = trainDAO.loadAll();
        for (Train train : trainList) {
            IState state = new DefaultState(factory);
            for (Fill fill : train.getFill()) {
                 state = state.handle(fill, logger);
            }
            if (!state.isValid()) {
                valid = false;
            }
        }

        return valid;
    }

    private interface IState {

        IState handle(final Fill fill, Logger logger);
        boolean isValid();
    }

    private class DefaultState implements IState {
        private DAOFactory factory;
        private boolean valid;

        public DefaultState(DAOFactory factory) {
            this.factory = factory;
            valid = true;
        }

        public DefaultState(DAOFactory factory, boolean valid) {
            this.factory = factory;
            this.valid = valid;
        }

        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals("EnterResource") && fill.getResource().getClass() == Platform.class) {
                return new PlatformState(factory, fill, valid);
            }
            return this;
        }

        public boolean isValid() {
            return valid;
        }
    }

    private class PlatformState implements IState {
        private DAOFactory factory;
        private Fill begin;
        private boolean valid;

        public PlatformState(DAOFactory factory, Fill fill, boolean valid) {
            this.factory = factory;
            this.begin = fill;
            this.valid = valid;
        }

        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals("ExitResource")) {
                int duration = fill.getDate().getTime() - begin.getDate().getTime();
                Duration maxDwellTime = factory.getParametersDAO().getParameters().getMaxDwellTime();
                if (duration > maxDwellTime.getTime()) {
                    logger.log(Level.INFO, "Maximum duration of use of a platform error : "
                            + fill.getTrain().getName() + " maximum use duration of "
                            + begin.getResource().getName() + " at " + begin.getDate().toString()
                            + " shoud be " + maxDwellTime.toString() + " ("
                            + new Duration(duration).toString() + " found).");
                    valid = false;
                }
                return new DefaultState(factory, valid);
            } else if (fill.getEvent().equals("Arrival")) {
                return new ArrivalState(factory, begin, fill, valid);
            } else if (fill.getEvent().equals("Departure")) {
                return new DepartureState(factory, begin, fill, valid);
            }
            return this;
        }

        public boolean isValid() {
            return valid;
        }
    }

    private class DepartureState implements IState {
        private DAOFactory factory;
        private Fill begin;
        private Fill departure;
        private boolean valid;

        public DepartureState(DAOFactory factory, final Fill fill, Fill departure, boolean valid) {
            this.factory = factory;
            begin = fill;
            this.valid = valid;
            this.departure = departure;
        }

        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals("ExitResource")) {
                int duration = fill.getDate().getTime() - begin.getDate().getTime();
                Departure dep = factory.getDepartureDAO().loadByName(departure.getComplement());
                if (dep != null) {
                    Duration maxDwellTime = dep.getMaxDwell();
                    if (duration > maxDwellTime.getTime()) {
                        logger.log(Level.INFO, "Maximum duration of use of a platform error : "
                                + fill.getTrain().getName() + " maximum use duration of "
                                + begin.getResource().getName() + " at " + begin.getDate().toString()
                                + " shoud be " + maxDwellTime.toString() + " ("
                                + new Duration(duration).toString() + " found).");
                        valid = false;
                    }
                } else {
                    logger.log(Level.INFO, "Maximum duration of use of a platform error : "
                            + fill.getTrain().getName() + " departure event is associated with "
                            + departure.getComplement() + " which is not a departure "
                            + "at " + departure.getDate().toString() + ".");
                    valid = false;
                }
                return new DefaultState(factory, valid);
            }
            return this;
        }

        public boolean isValid() {
            return valid;
        }
    }

    private class ArrivalState implements IState {
        private DAOFactory factory;
        private Fill begin;
        private Fill arrival;
        private boolean valid;

        public ArrivalState(DAOFactory factory, final Fill fill, Fill arrival, boolean valid) {
            this.factory = factory;
            begin = fill;
            this.arrival = arrival;
            this.valid = valid;
        }

        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals("ExitResource")) {
                int duration = fill.getDate().getTime() - begin.getDate().getTime();
                Arrival arr = fill.getTrain().getArrival();
                if (arr != null) {
                    Duration maxDwellTime = arr.getMaxDwell();
                    if (duration > maxDwellTime.getTime()) {
                        logger.log(Level.INFO, "Maximum duration of use of a platform error : "
                                + fill.getTrain().getName() + " maximum use duration of "
                                + begin.getResource().getName() + " at " + begin.getDate().toString()
                                + " shoud be " + maxDwellTime.toString() + " ("
                                + new Duration(duration).toString() + " found).");
                        valid = false;
                    }
                } else {
                    logger.log(Level.INFO, "Maximum duration of use of a platform error : "
                            + fill.getTrain().getName() + " is a train already present in the system, "
                            + "and therefore cannot be associated with an arrival "
                            + "at " + arrival.getDate().toString() + ".");
                    valid = false;
                }
                return new DefaultState(factory, valid);
            } else if (fill.getEvent().equals("Departure")) {
                return new ArrivalDepartureState(factory, begin, fill, arrival, valid);
            }
            return this;
        }

        public boolean isValid() {
            return valid;
        }
    }

    private class ArrivalDepartureState implements IState {
        private DAOFactory factory;
        private Fill begin;
        private boolean valid;
        private Fill departure;
        private Fill arrival;

        public ArrivalDepartureState(DAOFactory factory, final Fill fill, Fill departure, Fill arrival
                , boolean valid) {
            this.factory = factory;
            begin = fill;
            this.valid = valid;
            this.departure = departure;
            this.arrival = arrival;
        }

        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals("ExitResource")) {
                int duration = fill.getDate().getTime() - begin.getDate().getTime();
                Departure dep = factory.getDepartureDAO().loadByName(departure.getComplement());
                if (dep != null) {
                    Arrival arr = fill.getTrain().getArrival();
                    if (arr != null) {
                        Duration maxDwellTime;
                        if (dep.getMaxDwell().getTime() > arr.getMaxDwell().getTime()) {
                            maxDwellTime = dep.getMaxDwell();
                        } else {
                            maxDwellTime = arr.getMaxDwell();
                        }

                        if (duration > maxDwellTime.getTime()) {
                            logger.log(Level.INFO, "Maximum duration of use of a platform error : "
                                    + fill.getTrain().getName() + " maximum use duration of "
                                    + begin.getResource().getName() + " at " + begin.getDate().toString()
                                    + " shoud be " + maxDwellTime.toString() + " ("
                                    + new Duration(duration).toString() + " found).");
                            valid = false;
                        }
                    } else {
                        logger.log(Level.INFO, "Maximum duration of use of a platform error : "
                                + fill.getTrain().getName() + " is a train already present in the system, "
                                + "and therefore cannot be associated with an arrival "
                                + "at " + arrival.getDate().toString() + ".");
                        valid = false;
                    }
                } else {
                    logger.log(Level.INFO, "Maximum duration of use of a platform error : "
                            + fill.getTrain().getName() + " departure event is associated with "
                            + departure.getComplement() + " which is not a departure "
                            + "at " + departure.getDate().toString() + ".");
                    valid = false;
                }
                return new DefaultState(factory, valid);
            }
            return this;
        }

        public boolean isValid() {
            return valid;
        }
    }
}