// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Arrival;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.IndividualTrack;
import com.sncf.checker.bo.JointArrDep;
import com.sncf.checker.bo.JointArrDepEntry;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.ArrivalDAO;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.IndividualTrackDAO;

public class TrainOrderOnIndividualTrackConstraint implements Constraint {

    private final DAOFactory factory;

    public TrainOrderOnIndividualTrackConstraint(DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        boolean valid = true;
        IndividualTrackDAO trackDAO = factory.getIndividualTrackDAO();
        Logger logger = Logger.getLogger("logger");

        for (IndividualTrack track : trackDAO.loadAll()) {
            List<Fill> fillList = track.getFill();
            Collections.sort(fillList, new SortFillByDate());
            ArrayList<Move> moveList = new ArrayList<Move>();

            for (int i = 0; i < fillList.size(); i++) {
                Fill f1 = fillList.get(i);
                for (int j = i + 1; j < fillList.size(); j++) {
                    Fill f2 = fillList.get(j);
                    if (f1.getDate().getTime() != f2.getDate().getTime()) {
                        break;
                    }

                    if (f1.getEvent().equals(f2.getEvent())
                            && (f1.getEvent().equals("EnterResource")
                                || f1.getEvent().equals("ExitResource"))
                            && (f1.getGate().equals(f2.getGate()))
                            && !isAssembled(f1, f2)
                            && f1.getDate().getTime() != Datetime.MAX.getTime()
                            && f1.getDate().getTime() != Datetime.MIN.getTime()) {
                        logger.log(Level.INFO, "Train order on individual track error : "
                                + f1.getTrain().getName() + " (" + f1.getDate().toString() + " " + f1.getGate()
                                + ") is in conflict with "
                                + f2.getTrain().getName() + " (" + f2.getDate().toString() + " " + f2.getGate()
                                + ") at " + track.getName() + ".");
                        valid = false;
                    }
                }
            }

            for (int i = 0; i < fillList.size(); i++) {
                Fill beg = fillList.get(i);
                Fill end = null;
                if (beg.getEvent().equals("EnterResource")) {
                    for (int j = i + 1; j < fillList.size(); j++) {
                        Fill fill = fillList.get(j);
                        if (fill.getTrain() == beg.getTrain()) {
                            if (fill.getEvent().equals("ExitResource")) {
                                end = fill;
                                break;
                            }
                        }
                    }
                    Move move = createMove(beg, end);
                    if (move.isValid()) {
                        moveList.add(move);
                    }
                }

            }

            for (int i = 0; i < moveList.size(); i++) {
                for (int j = i + 1; j < moveList.size(); j++) {
                    Move m1 = moveList.get(i);
                    Move m2 = moveList.get(j);
                    if (m2.getBeg().getTime() > m1.getEnd().getTime()) {
                        break;
                    }

                    if (detectConflict(m1, m2)) {
                        logger.log(Level.INFO, "Train order on individual track error : "
                                + m1.getTrain().getName() + " (" + m1.getBeg().toString() + " " + m1.getOriginSide()
                                + " - " +  m1.getEnd().toString() + " " + m1.getDestinationSide()
                                + ") is in conflict with "
                                + m2.getTrain().getName() + " (" + m2.getBeg().toString() + " " + m2.getOriginSide()
                                + " - " +  m2.getEnd().toString() + " " + m2.getDestinationSide()
                                + ") at " + track.getName() + ".");
                        valid = false;
                    }
                }
            }
        }

        return valid;
    }

    private Move createMove(Fill beg, Fill end) {
        Move move = new Move();
        if (beg != null) {
            move.setTrain(beg.getTrain());
            move.setBeg(beg.getDate());
            if (beg.getGate().length() >= 1) {
                move.setOriginSide(beg.getGate().substring(0, 1));
            }
        }
        if (end != null) {
            move.setEnd(end.getDate());
            if (end.getGate().length() >= 1) {
                move.setDestinationSide(end.getGate().substring(0, 1));
            } else {
                move.setDestinationSide(move.getOriginSide());
            }
        }
        return move;
    }

    private boolean detectConflict(Move m1, Move m2) {

        if (m1.getBeg().getTime() == m2.getBeg().getTime()
                && m1.getOriginSide().equals(m2.getOriginSide())) {
            return false;
        }
        if (m1.getOriginSide().equals("A")) {
            if (m1.getDestinationSide().equals("A")) {
                if (m2.getOriginSide().equals("A")) {
                    if (m2.getDestinationSide().equals("A")) {
                        if (m2.getEnd().getTime() > m1.getEnd().getTime()) {
                            return true;
                        }
                    } else {
                        return true;
                    }
                } else {
                    if (m2.getDestinationSide().equals("A")) {
                        if (m2.getEnd().getTime() < m1.getEnd().getTime()) {
                            return true;
                        }
                    }
                }
            }
            else {
                if (m2.getOriginSide().equals("A")) {
                    if (!m2.getDestinationSide().equals("A")) {
                        if (m2.getEnd().getTime() < m1.getEnd().getTime()) {
                            return true;
                        }
                    }
                } else {
                    if (m2.getDestinationSide().equals("A")) {
                        return true;
                    } else {
                        if (m2.getEnd().getTime() > m1.getEnd().getTime()) {
                            return true;
                        }
                    }
                }
            }
        }
        else {
            if (m1.getDestinationSide().equals("A")) {
                if (m2.getOriginSide().equals("A")) {
                    if (m2.getDestinationSide().equals("A")) {
                        if (m2.getEnd().getTime() > m1.getEnd().getTime()) {
                            return true;
                        }
                    } else {
                        return true;
                    }
                } else {
                    if (m2.getDestinationSide().equals("A")) {
                        if (m2.getEnd().getTime() < m1.getEnd().getTime()) {
                            return true;
                        }
                    }
                }
            }
            else {
                if (m2.getOriginSide().equals("A")) {
                    if (!m2.getDestinationSide().equals("A")) {
                        if (m2.getEnd().getTime() < m1.getEnd().getTime()) {
                            return true;
                        }
                    }
                } else {
                    if (m2.getDestinationSide().equals("A")) {
                        if (m2.getEnd().getTime() < m1.getEnd().getTime()) {
                            return true;
                        }
                    } else {
                        if (m2.getEnd().getTime() > m1.getEnd().getTime()) {
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }

    private boolean isAssembled(Fill f1, Fill f2) throws ROADEFException {
        ArrivalDAO arrivalDAO = factory.getArrivalDAO();
        ArrayList<String> assembledTrains = new ArrayList<String>();
        Train train = f1.getTrain();
        Arrival arrival = train.getArrival();
        if (arrival != null) {
            JointArrDep joint = arrival.getJointArrDep();
            if (joint != null) {
                for (JointArrDepEntry entry : joint.getEntryList()) {
                    Arrival jointArrival = arrivalDAO.loadByName(entry.getArrDep());
                    assembledTrains.add(jointArrival.getTrain().getName());
                }
            } else {
                assembledTrains.add(train.getName());
            }
        } else {
            assembledTrains.add(train.getName());
        }

        for (Fill fill : train.getFill()) {
            if (fill == f1) {
                break;
            }
            if (fill.getEvent().equals("EndJunction") || fill.getEvent().equals("EndDisjunction")) {
                assembledTrains = new ArrayList<String>(Arrays.asList(fill.getComplement().split("\\+")));
            }
        }


        if (assembledTrains.contains(f2.getTrain().getName())) {
            return true;
        }

        return false;
    }

    private class Move implements Comparable<Move> {

        public Move() {
        }

        private Train train;

        public Train getTrain() {
            return train;
        }

        public void setTrain(Train train) {
            this.train = train;
        }

        private Datetime beg;

        public Datetime getBeg() {
            return beg;
        }

        public void setBeg(Datetime beg) {
            this.beg = beg;
        }

        private Datetime end;

        public Datetime getEnd() {
            return end;
        }

        public void setEnd(Datetime end) {
            this.end = end;
        }

        String originSide;

        public String getOriginSide() {
            return originSide;
        }

        public void setOriginSide(String originSide) {
            this.originSide = originSide;
        }

        String destinationSide;

        public String getDestinationSide() {
            return destinationSide;
        }

        public void setDestinationSide(String destinationSide) {
            this.destinationSide = destinationSide;
        }

        public boolean isValid() {
            if (beg == null
                    || end == null
                    || train == null
                    || originSide == null
                    || destinationSide == null) {
                return false;
            } else {
                if (!(originSide.equals("A") || originSide.equals("B"))
                        || !(destinationSide.equals("A") || destinationSide.equals("B"))) {
                    return false;
                }
            }
            return true;
        }

        public int compareTo(Move arg0) {
            if (beg.getTime() < arg0.beg.getTime()) {
                return -1;
            } else if (beg.getTime() > arg0.beg.getTime()) {
                return 1;
            } else {
                return 0;
            }

        }
    }
}
