// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.bo;

import java.util.ArrayList;
import java.util.List;

import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.Duration;
import com.sncf.checker.io.PlatformDAO;
import com.sncf.checker.io.TrainCategoryDAO;

public class Departure {

    /** dao for compatible platform list lazy loading. */
    private PlatformDAO platformDAO;
    /** dao for compatible category list lazy loading. */
    private TrainCategoryDAO categoryDAO;

    /**
     * Constructor.
     */
    public Departure() {
        prefPlat = new ArrayList<Platform>();
        sequenceList = new ArrayList<ArrDepSequence>();
        compCatDep = new ArrayList<TrainCategory>();
    }

    /**
     * lazy-loading version constructor.
     *
     * @param occupationDAO
     * @param fillDAO
     */
    public Departure(final PlatformDAO platformDAO, final TrainCategoryDAO categoryDAO) {
        this.platformDAO = platformDAO;
        sequenceList = new ArrayList<ArrDepSequence>();
        this.categoryDAO = categoryDAO;
    }

    /** name. */
    private String name;

    /**
     * Get the name.
     *
     * @return name
     */
    public final String getName() {
        return name;
    }

    /**
     * Set the name.
     *
     * @param name String to set
     */
    public final void setName(final String name) {
        this.name = name;
    }

    /** departure time. */
    private Datetime depTime;

    /**
     * Get the departure time.
     *
     * @return departure time
     */
    public final Datetime getDepTime() {
        return depTime;
    }

    /**
     * Set the departure time.
     *
     * @param depTime departure time
     */
    public final void setDepTime(final Datetime depTime) {
        this.depTime = depTime;
    }

    /** departure sequence. */
    private ArrayList<ArrDepSequence> sequenceList;

    /**
     * Get the departure sequence.
     *
     * @return departure sequence
     */
    public final List<ArrDepSequence> getDepSequence() {
        return sequenceList;
    }

    /**
     * Set the departure sequence.
     *
     * @param sequence departure sequence
     */
    public final void addDepSequence(final ArrDepSequence sequence) {
        sequenceList.add(sequence);
    }

    /** prefered platforms. */
    private ArrayList<Platform> prefPlat;

    /**
     * Get the prefered platforms.
     *
     * @return platform list
     */
    public final List<Platform> getPreferedPlatforms() {
        if (prefPlat == null) {
            prefPlat = new ArrayList<Platform>(platformDAO.loadCompatible(name));
        }
        return prefPlat;
    }

    /**
     * Add a prefered platform.
     *
     * @param platform to add
     */
    public final void addPreferedPlatform(final Platform platform) {
        if (prefPlat == null) {
            prefPlat = new ArrayList<Platform>(platformDAO.loadCompatible(name));
        }
        prefPlat.add(platform);
    }

    /** ideal dwell time. */
    private Duration idealDwell;

    /**
     * Get the ideal dwell time.
     *
     * @return ideal dwell time
     */
    public final Duration getIdealDwell() {
        return idealDwell;
    }

    /**
     * Set the ideal dwell time.
     *
     * @param duration ideal dwell time
     */
    public final void setIdealDwell(final Duration duration) {
        this.idealDwell = duration;
    }

    /** max dwell time. */
    private Duration maxDwell;

    /**
     * Get the max dwell time.
     *
     * @return remaining time
     */
    public final Duration getMaxDwell() {
        return maxDwell;
    }

    /**
     * Set the max dwell time.
     *
     * @param maxDwell max dwell time
     */
    public final void setMaxDwell(final Duration duration) {
        this.maxDwell = duration;
    }

    /** remaining distance before maintenance. */
    private int reqDBM;

    /**
     * Get the remaining distance before maintenance.
     *
     * @return remaining distance
     */
    public final int getReqDBM() {
        return reqDBM;
    }

    /**
     * Set remaining distance before maintenance.
     *
     * @param remDBM remaining distance
     */
    public final void setReqDBM(final int reqDBM) {
        this.reqDBM = reqDBM;
    }

    /** required time before maintenance. */
    private Duration reqTBM;

    /**
     * Get the required time before maintenance.
     *
     * @return required time
     */
    public final Duration getReqTBM() {
        return reqTBM;
    }

    /**
     * Set required time before maintenance.
     *
     * @param remDBM remaining time
     */
    public final void setReqTBM(final Duration duration) {
        this.reqTBM = duration;
    }

    /** compatible categories. */
    private ArrayList<TrainCategory> compCatDep;

    /**
     * Get the compatible categories.
     *
     * @return compatible categories
     */
    public final List<TrainCategory> getCompCatDep() {
        if (compCatDep == null) {
            compCatDep = new ArrayList<TrainCategory>(categoryDAO.loadByDeparture(name));
        }
        return compCatDep;
    }

    /**
     * Add a compatible category.
     *
     * @param category compatible category to add
     */
    public final void addCompCatDep(final TrainCategory category) {
        if (compCatDep == null) {
            compCatDep = new ArrayList<TrainCategory>(categoryDAO.loadByDeparture(name));
        }
        compCatDep.add(category);
    }

    /** name. */
    private Arrival linkedArr;

    /**
     * Get the name.
     *
     * @return name
     */
    public final Arrival getLinkedArr() {
        return linkedArr;
    }

    /**
     * Set the name.
     *
     * @param name String to set
     */
    public final void setLinkedArr(final Arrival linkedArr) {
        this.linkedArr = linkedArr;
        if (linkedArr != null) {
            if (linkedArr.getLinkedDep() != this) {
                linkedArr.setLinkedDep(this);
            }
        }
    }


    JointArrDep jointArrDep;

    public JointArrDep getJointArrDep() {
        return jointArrDep;
    }

    public void setJointArrDep(JointArrDep jointArrDep) {
        this.jointArrDep = jointArrDep;
    }
}
