// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.ArrDepSequence;
import com.sncf.checker.bo.Arrival;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.Duration;

/**
 * The data source is a tabulation separated file containing the arrivals.
 *
 * @author YDS
 *
 */
public class ArrivalFlatFileDAO extends FlatFileDAO implements ArrivalDAO {

    /** scenario file path. */
    private final String scenario;
    /** indexed by name train map of the scenario. */
    private Map<String, Arrival> cache = null;
    private Map<String, Arrival> trainIndexedCache = null;

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public ArrivalFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** ArrDepSequence DAO. */
    private ArrDepSequenceDAO arrDepSequenceDAO;

    /**
     * ArrDepSequence DAO setter.
     *
     * @param dao ArrDepSequence DAO
     */
    public void setArrDepSequenceDAO(final ArrDepSequenceDAO dao) {
        this.arrDepSequenceDAO = dao;
    }

    /** Platform DAO. */
    private PlatformDAO platformDAO;

    /**
     * Platform DAO setter.
     *
     * @param dao Platform DAO
     */
    public void setPlatformDAO(final PlatformDAO dao) {
        this.platformDAO = dao;
    }

    /** Train DAO. */
    private TrainDAO trainDAO;

    /**
     * Train DAO setter.
     *
     * @param dao Train DAO
     */
    public void setTrainDAO(final TrainDAO dao) {
        this.trainDAO = dao;
    }

    /** Departure DAO. */
    private DepartureDAO departureDAO;

    /**
     * Departure DAO setter.
     *
     * @param dao Departure DAO
     */
    public void setDepartureDAO(final DepartureDAO dao) {
        this.departureDAO = dao;
    }

    /** ArrDepSequence DAO. */
    private JointArrDepDAO jointArrDepDAO;

    /**
     * ArrDepSequence DAO setter.
     *
     * @param dao ArrDepSequence DAO
     */
    public void setJointArrDepDAO(final JointArrDepDAO dao) {
        this.jointArrDepDAO = dao;
    }

    /**
     * Load a train by name.
     *
     * @param name the searched train name
     * @return train object, null if not found
     */
    public Arrival loadByName(final String name) {
        initialize();
        Arrival entity = cache.get(name);
        return entity;
    }

    /**
     * Load an arrival by train name.
     *
     * @param name the searched train name
     * @return arrival, null if not found
     */
    public Arrival loadByTrain(final String name) {
        initialize();
        Arrival entity = trainIndexedCache.get(name);
        return entity;
    }

    /**
     * Load all trains.
     *
     * @return train list
     */
    public List<Arrival> loadAll() {
        initialize();
        ArrayList<Arrival> list = new ArrayList<Arrival>();
        list.addAll(cache.values());
        return list;
    }

    /**
     * Extract all trains from arrivals.csv in zip file.
     */
    private void initialize() {
        if (cache == null || trainIndexedCache == null) {
            cache = new TreeMap<String, Arrival>();
            trainIndexedCache = new TreeMap<String, Arrival>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "arrivals.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));
                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    Arrival temp;

                    temp = deserialize(splits);
                    cache.put(temp.getName(), temp);
                    trainIndexedCache.put(splits[1], temp);
                    temp.setTrain(trainDAO.loadByName(splits[1]));
                    List<ArrDepSequence> sequenceList = arrDepSequenceDAO.loadByArrDep(splits[3]);
                    for (ArrDepSequence seq : sequenceList) {
                        temp.addArrSequence(seq);
                    }
                    temp.setLinkedDep(departureDAO.loadByName(splits[5]));
                    temp.setJointArrDep(jointArrDepDAO.loadByName(splits[4]));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a Train object.
     *
     * @param entry data extracted from the file
     * @return train
     */
    private Arrival deserialize(final String[] raw) {
        Arrival entity = new Arrival(platformDAO);

        entity.setName(raw[0]);
        entity.setArrTime(new Datetime(raw[2]));

        entity.setIdealDwell(new Duration(raw[7]));
        entity.setMaxDwell(new Duration(raw[8]));
        return entity;
    }
}
