// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.ArrDepSequence;
import com.sncf.checker.bo.Departure;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.Duration;

/**
 * The data source is a tabulation separated file containing the arrivals.
 *
 * @author YDS
 *
 */
public class DepartureFlatFileDAO extends FlatFileDAO implements DepartureDAO {

    /** scenario file path. */
    private final String scenario;
    /** indexed by name train map of the scenario. */
    private Map<String, Departure> cache = null;

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public DepartureFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** ArrDepSequence DAO. */
    private ArrDepSequenceDAO arrDepSequenceDAO;

    /**
     * ArrDepSequence DAO setter.
     *
     * @param dao ArrDepSequence DAO
     */
    public void setArrDepSequenceDAO(final ArrDepSequenceDAO dao) {
        this.arrDepSequenceDAO = dao;
    }

    /** Platform DAO. */
    private PlatformDAO platformDAO;

    /**
     * Platform DAO setter.
     *
     * @param dao Platform DAO
     */
    public void setPlatformDAO(final PlatformDAO dao) {
        this.platformDAO = dao;
    }

    /** TrainCategory DAO. */
    private TrainCategoryDAO trainCategoryDAO;

    /**
     * TrainCategory DAO setter.
     *
     * @param dao TrainCategory DAO
     */
    public void setTrainCategoryDAO(final TrainCategoryDAO dao) {
        this.trainCategoryDAO = dao;
    }

    /** ArrDepSequence DAO. */
    private JointArrDepDAO jointArrDepDAO;

    /**
     * ArrDepSequence DAO setter.
     *
     * @param dao ArrDepSequence DAO
     */
    public void setJointArrDepDAO(final JointArrDepDAO dao) {
        this.jointArrDepDAO = dao;
    }

    /**
     * Load a train by name.
     *
     * @param name the searched train name
     * @return train object, null if not found
     */
    public Departure loadByName(final String name) {
        initialize();
        Departure entity = cache.get(name);
        return entity;
    }

    /**
     * Load all trains.
     *
     * @return train list
     */
    public List<Departure> loadAll() {
        initialize();
        ArrayList<Departure> list = new ArrayList<Departure>();
        list.addAll(cache.values());
        return list;
    }

    /**
     * Extract all trains from departures.csv in zip file.
     */
    private void initialize() {
        if (cache == null) {
            cache = new TreeMap<String, Departure>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "departures.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";", -1);
                    Departure temp;

                    temp = deserialize(splits);
                    cache.put(temp.getName(), temp);
                     List<ArrDepSequence> sequenceList = arrDepSequenceDAO.loadByArrDep(splits[2]);
                    for (ArrDepSequence seq : sequenceList) {
                        temp.addDepSequence(seq);
                    }
                    temp.setJointArrDep(jointArrDepDAO.loadByName(splits[3]));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a Train object.
     *
     * @param entry data extracted from the file
     * @return train
     */
    private Departure deserialize(final String[] raw) {
        Departure entity = new Departure(platformDAO, trainCategoryDAO);

        entity.setName(raw[0]);
        entity.setDepTime(new Datetime(raw[1]));
        entity.setIdealDwell(new Duration(raw[4]));
        entity.setMaxDwell(new Duration(raw[5]));
        entity.setReqDBM(Integer.parseInt(raw[6]));
        entity.setReqTBM(new Duration(raw[7]));

        return entity;
    }
}
