// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Resource;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.common.TreeMultiMap;

/**
 * The data source is a tabulation separated file containing the Fill objects.
 *
 * @author YDS
 *
 */
public class FillFlatFileDAO implements FillDAO {

    /** solution file path. */
    private  String solution;
    private int line;
    private List<Fill> cache = null;
    private TreeMultiMap<String, Fill> trainIndexedCache = null;
    private TreeMultiMap<String, Fill> occupationIndexedCache = null;

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public FillFlatFileDAO(final String scenario, final String solution) {
        this.solution = solution;
        line = 1;
    }

    /** Train DAO. */
    private TrainDAO trainDAO;

    /**
     * Train DAO setter.
     *
     * @param dao Train DAO
     */
    public void setTrainDAO(final TrainDAO dao) {
        this.trainDAO = dao;
    }

    /** Resource DAO. */
    private ResourceDAO resourceDAO;

    /**
     * Resource DAO setter.
     *
     * @param dao Resource DAO
     */
    public void setResourceDAO(final ResourceDAO dao) {
        this.resourceDAO = dao;
    }

    public List<Fill> loadAll() throws ROADEFException {
        initialize();
        ArrayList<Fill> list = new ArrayList<Fill>();
        list.addAll(cache);
        return list;
    }

    /**
     * load a Fill list by train name.
     *
     * @param name train name
     * @return Fill list
     * @throws ROADEFException
     */
    public List<Fill> loadByTrain(final String name) throws ROADEFException {
        initialize();
        return trainIndexedCache.get(name);
    }

    /**
     * load a Fill list by occupation name.
     *
     * @param name occupation name
     * @return Fill list
     * @throws ROADEFException
     */
    public List<Fill> loadByOccupation(final String name) throws ROADEFException {
        initialize();
        return occupationIndexedCache.get(name);
    }

    /**
     * Extract all Fill from Solution.csv.
     * @throws ROADEFException
     */
    private void initialize() throws ROADEFException {
        if (trainIndexedCache == null || occupationIndexedCache == null || cache == null) {
            trainIndexedCache = new TreeMultiMap<String, Fill>();
            occupationIndexedCache = new TreeMultiMap<String, Fill>();
            cache = new ArrayList<Fill>();

            BufferedReader bufferedReader = null;

            try {
                String current;

                bufferedReader = new BufferedReader(new FileReader(solution));
                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    line++;
                    String[] splits = current.split(";", -1);
                    Fill entity;
                    if (splits.length != 7) {
                        throw new ROADEFException("Error while reading " + solution + " (line "
                                + line + ") : 6 column expected, " + (splits.length - 1) + " found.");
                    }
                    entity = deserialize(splits);
                    trainIndexedCache.put(splits[0], entity);
                    occupationIndexedCache.put(splits[3], entity);
                    cache.add(entity);
                    Train train = trainDAO.loadByName(splits[0]);
                    if (train == null) {
                        throw new ROADEFException("Error while reading " + solution + " (line "
                                + line + ") : " + splits[0] + " is not a train.");
                    }
                    entity.setTrain(train);
                    Resource resource = resourceDAO.loadByName(splits[3]);
                    if (resource == null) {
                        throw new ROADEFException("Error while reading " + solution + " (line "
                                + line + ") : " + splits[3] + " is not a resource.");
                    }
                    entity.setResource(resourceDAO.loadByName(splits[3]));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a Fill object.
     *
     * @param entry data extracted from the file
     * @return train
     * @throws ROADEFException
     */
    private Fill deserialize(final String[] raw) throws ROADEFException {

        Fill entity = new Fill();

        entity.setDate(new Datetime(raw[1]));
        entity.setEvent(raw[2]);
        entity.setGate(raw[4]);
        entity.setComplement(raw[5]);

        return entity;
    }

}
