// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.TrackGroup;
import com.sncf.checker.common.Duration;

/**
 * The data source is a tabulation separated file containing the track groups.
 *
 * @author YDS
 *
 */
public class TrackGroupFlatFileDAO extends FlatFileDAO implements TrackGroupDAO {

    /** scenario file path. */
    private final String scenario;
    /** indexed by name track group map of the scenario. */
    private Map<String, TrackGroup> cache = null;

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public TrackGroupFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** TrainCategory DAO. */
    private TrainCategoryDAO trainCategoryDAO;

    /**
     * TrainCategory DAO setter.
     *
     * @param dao TrainCategory DAO
     */
    public void setTrainCategoryDAO(final TrainCategoryDAO dao) {
        this.trainCategoryDAO = dao;
    }

    /** Fill DAO. */
    private FillDAO fillDAO;

    /**
     * Fill DAO setter.
     *
     * @param dao Fill DAO
     */
    public void setFillDAO(final FillDAO dao) {
        this.fillDAO = dao;
    }

    /** Gate DAO. */
    private GateDAO gateDAO;

    /**
     * Gate DAO setter.
     *
     * @param dao Gate DAO
     */
    public void setGateDAO(final GateDAO dao) {
        this.gateDAO = dao;
    }

    /**
     * Load a track group by name.
     *
     * @param name the searched track group name
     * @return track group object, null if not found
     */
    public TrackGroup loadByName(final String name) {
        initialize();
        TrackGroup entity = cache.get(name);
        return entity;
    }

    /**
     * Load all track groups.
     *
     * @return track group list
     */
    public List<TrackGroup> loadAll() {
        initialize();
        ArrayList<TrackGroup> list = new ArrayList<TrackGroup>();
        list.addAll(cache.values());
        return list;
    }

    /**
     * Extract all track groups from trackGroups.csv in zip file.
     */
    private void initialize() {
        if (cache == null) {
            cache = new TreeMap<String, TrackGroup>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "trackGroups.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    TrackGroup temp;

                    temp = deserialize(splits);
                    cache.put(temp.getName(), temp);
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a TrackGroup object.
     *
     * @param entry data extracted from the file
     * @return track group
     */
    private TrackGroup deserialize(final String[] raw) {
        TrackGroup entity = new TrackGroup(gateDAO, fillDAO, trainCategoryDAO);

        entity.setName(raw[0]);
        entity.setTravelTime(new Duration(raw[1]));
        entity.setHeadway(new Duration(raw[2]));

        return entity;
    }
}
